<?php
namespace Ig\IgChart\Domain\Model;

use \TYPO3\CMS\Core\Utility\GeneralUtility;

class Chart extends \TYPO3\CMS\Extbase\DomainObject\AbstractEntity
{

    /**
     * type
     *
     * @var string
     */
    protected $id = '';

    /**
     * type
     *
     * @var string
     */
    protected $type = 'bar';

    /**
     * labels
     *
     * @var array
     */
    protected $labels = [];

    /**
     * data
     *
     * @var array
     */
    protected $data = [];
    /**
     * colors
     *
     * @var array
     */
    protected $colors = ['#e0440e', '#e6693e', '#ec8f6e', '#f3b49f', '#f6c7b6'];

    public function __construct()
    {
        $this->chartUtility = GeneralUtility::makeInstance(\Ig\IgChart\Utility\ChartUtility::class);
        $this->id = $this->chartUtility->getId();
    }

    /**
     * Returns the id
     *
     * @return string $id
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Sets the id
     *
     * @param string $id
     * @return void
     */
    public function setId(string $id)
    {
        $this->id = $id;
    }

    /**
     * Returns the type
     *
     * @return string $type
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * Sets the type
     *
     * @param string $type
     * @return void
     */
    public function setType(string $type)
    {
        $this->type = $type;
    }

    /**
     * Returns the labels
     *
     * @return aray $labels
     */
    public function getLabels()
    {
        return $this->labels;
    }

    /**
     * Sets the labels
     *
     * @param array $labels
     * @return void
     */
    public function setLabels(array $labels)
    {
        $this->labels = $labels;
    }
    public function addLabel($label)
    {
        $this->labels[] = $label;
    }

    /**
     * Returns the datasets
     *
     * @return aray $datasets
     */
    public function getDatasets()
    {
        return $this->datasets;
    }

    /**
     * Sets the datasets
     *
     * @param array $datasets
     * @return void
     */
    public function setDatasets(array $datasets)
    {
        $this->datasets = $datasets;
    }

    public function addDataset(array $dataset)
    {
        $this->datasets[] = $dataset;
    }

    /**
     * Returns the colors
     *
     * @return aray $colors
     */
    public function getColors()
    {
        return $this->colors;
    }
    public function getColor($nr)
    {
        return $this->colors[$nr];
    }
    public function getColorsRgba($alpha = 1)
    {
        $colors = [];
        foreach ($this->colors as $color) {
            $colors[] = $this->chartUtility->hex2rgba($color, $alpha);
        }
        return $colors;
    }
    public function getColorRgba($nr, $alpha)
    {
        return $this->chartUtility->hex2rgba($this->colors[$nr], $alpha);
    }

    /**
     * Sets the colors
     *
     * @param array $colors
     * @return void
     */
    public function setColors(array $colors)
    {
        $this->colors = $colors;
    }

    /**
     * Prepare data from [ x=> [y,y2], x=> [y,y2], x=> [y,y2]]
     *
     * @param array $dataRecords
     * @param int $dataLimit
     * @return void
     */
    public function setDatasetsAndLabels($dataRecords, int $dataLimit = 0)
    {
        $datasets = [];
        $datasetData = [];
        $index = 0;
        foreach ($dataRecords as $label => $data) {            
            $this->addLabel($label);
            foreach ($data as $key => $d) {
                if (!isset($dataset[$key])) {
                    $dataset[$key] = new \stdClass;
                    $dataset[$key]->data = [];
                }
                $dataset[$key]->data[] = $d;
            }
            $index++;
            if ($dataLimit > 0 &&  $index >= $dataLimit) {
                break;
            }
        }
        $this->setDatasets($datasets);
    }

    /**
     * Prepare data from [[x,y,y2],[x,y,y2],[x,y,y2]]
     *
     * @param array $chartData
     * @param string $x
     * @param mixed $y (array oder string)
     * @return void
     */
    public function setDatasetsAndLabelsFromRecords($dataRecords, $x, $y, $colorAttribute = null)
    {
        $datasets = [];
        // y ist Array -> Mehrere Datenreihen
        if (is_array($y)) {
            $datasetData = [];
            foreach ($y as $label => $attribute) {
                $datasetData[$label] = [];
            }
            foreach ($dataRecords as $nr => $data) {
                $this->addLabel($data[$x]);
                foreach ($y as $label => $attribute) {
                    $datasetData[$label][] = (float) $data[$attribute];
                }
            }
            foreach ($datasetData as $label => $data) {
                $dataset = new \stdClass;
                $dataset->label = $label;
                $dataset->data = $data;
                $datasets[] = $dataset;
            }
        } else {
            // y ist nur ein Attributename
            $colors = [];
            $datasetData = [];
            foreach ($dataRecords as $nr => $data) {
                $this->addLabel($data[$x]);
                $datasetData[] = $data[$y];
                if ($colorAttribute) {
                    $colors[] = $data[$colorAttribute];
                }
            }
            $dataset = new \stdClass;
            $dataset->data = $datasetData;
            $datasets[] = $dataset;
            if (!empty($colors)) {
                $this->setColors($colors);
            }
        }
        $this->setDatasets($datasets);
    }

    public function datasetsAppend()
    {
        if (strpos($color, 'rgba') !== false) {
            $alpha = $this->chartUtility->getAlphaFromRgba($color);
        }
        $alpha = ($alpha) ? $alpha : 1;
        $alphaHover = ($alpha) ? $alpha - 0.1 : 0.7;
        $specialAlpha = ($alpha) ? $alpha - 0.1 : 0.8;
        $borderAlpha = ($alpha) ? $alpha - 0.1 : 0.7;

        $datasets = [];
        $count = 0;
        foreach ($this->getDatasets() as $dataset) {
            switch ($this->type) {
                case 'pie':
                case 'doughnut':
                    $dataset->backgroundColor = $this->getColorsRgba($alpha);
                    $dataset->hoverBackgroundColor = $this->getColorsRgba($alphaHover);
                    break;

                case 'polarArea':
                    break;

                case 'bar':
                case 'horizontalBar':
                    $dataset->backgroundColor = $this->getColorRgba($count, $alpha);
                    $dataset->borderColor = $this->getColorRgba($count, $borderAlpha);
                    $dataset->hoverBackgroundColor = $this->getColorRgba($count, $specialAlpha);
                    $dataset->hoverBorderColor = $this->getColorRgba($count, $specialAlpha);
                    break;

                case 'line':
                case 'radar':
                    $dataset->fill = false;
                    $dataset->backgroundColor = $this->getColorRgba($count, $alpha);
                    $dataset->borderColor = $this->getColorRgba($count, $alpha);
                    $dataset->pointBackgroundColor = $this->getColorRgba($count, $specialAlpha);
                    $dataset->pointBorderColor = '#FFF';
                    $dataset->pointHoverBackgroundColor = '#FFF';
                    $dataset->pointHoverBorderColor = $this->getColorRgba($count, $specialAlpha);
                    break;
                default:
                    break;
            }
            $count++;
            $datasets[] = $dataset;
        }
        return $datasets;
    }

    public function getData()
    {
        $ret = new \stdClass;

        $ret->datasets = $this->datasetsAppend();
        $ret->labels = $this->getLabels();
        return $ret;
    }

    public function getDataJson()
    {
        return json_encode($this->getData());
    }

    public function getHtml($arguments)
    {
        // Render HTML and JS
        /*
        $html = '';
        $html .= '<div class="ig-chart-container chart-' . $this->getType() . '">';
        $html .= '<div class="canvasContainer ' . $extraClasses . '">';
        $html .= '<canvas id="igChart'.$this->chartUtility->getId().'" width="100" height="100" />';
        $html .= '</div>';
        $html .= '</div>';
        $js="var igChart".$this->chartUtility->getId()." = new Chart('igChart".$this->chartUtility->getId()."',{
        type: '".$this->getType()."',
        data: ".$this->getDataJson()."
        });";
        $pageRenderer=GeneralUtility::makeInstance(\TYPO3\CMS\Core\Page\PageRenderer::class);
        $pageRenderer->addJsFooterInlineCode( 'ig-chart-' . $this->chartUtility->getId(), $js);
        return $html;
         */

        // Generate extra classes
        $extraClasses = '';
        if ($arguments['showLegend']) {
            $extraClasses .= 'hasLegend ';
        }
        $extraClasses = rtrim($extraClasses);

        // Generate extra data tags
        $extraData = 'data-yscalesuffix="' . $arguments['yScaleSuffix'] . '" ';
        $extraData .= 'data-xscalesuffix="' . $arguments['xScaleSuffix'] . '" ';
        $extraData = rtrim($extraData);

        // Get the ratio of the chart
        $ratio = explode('-', $arguments['ratio']);

        $html .= '<div class="frame frame-default ig-chart-container chart-' . $this->getType() . '">';
        $html .= '<div class="canvasContainer ' . $extraClasses . '">';
        $html .= '<canvas
                        data-offset="-300"
                        width="' . $ratio[0] . '"
                        height="' . $ratio[1] . '"
                        style=""
                        class="ig-chart" ' . $extraData . '
                        data-showlegend="' . $arguments['showLegend'] . '"
                        data-charttype="' . $this->getType() . '"
                        data-chart="' . htmlentities($this->getDataJson(), ENT_QUOTES, 'UTF-8') . '"
                        data-cutoutpercentage="' . $arguments['cutoutPercentage'] . '"
                        data-showdoughnuttotal="' . $arguments['showDoughnutTotal'] . '"
                        data-doughnuttotalsuffix="' . $arguments['doughnutTotalSuffix'] . '"
                        data-showxscale="' . $arguments['showXScale'] . '"
                        data-showyscale="' . $arguments['showYScale'] . '"
                        data-ystepsize="' . $arguments['yStepSize'] . '"
                        data-yscalemax="' . $arguments['yScaleMax'] . '"
                        data-yscalemin="' . $arguments['yScaleMin'] . '"
                        data-xstepsize="' . $arguments['xStepSize'] . '"
                        data-xscalemax="' . $arguments['xScaleMax'] . '"
                        data-xscalemin="' . $arguments['xScaleMin'] . '"
                        data-xreverse="' . $arguments['xReverse'] . '"
                        data-yreverse="' . $arguments['yReverse'] . '"
                        data-xscaleposition="' . $arguments['xScalePosition'] . '"
                        data-yscaleposition="' . $arguments['yScalePosition'] . '"
                        data-scalefontfamily="' . $arguments['scaleFontFamily'] . '"
                        data-scalefontcolor="' . $arguments['scaleFontColor'] . '"
                        data-scalefontsize="' . $arguments['scaleFontSize'] . '"
                        data-scalefontstyle="' . $arguments['scaleFontStyle'] . '"
                        data-animationduration="' . $arguments['animationDuration'] . '"
                        data-animationeasing="' . $arguments['animationEasing'] . '"
                        data-animationdelay="' . $arguments['animationDelay'] . '"
                        data-additional-options="' . htmlspecialchars(json_encode($arguments['additionalOptions'])) . '"
                        >';

        if (isset($arguments['fallbackImg']) && is_integer($arguments['fallbackImg'])) {
            $fileRepository = GeneralUtility::makeInstance(\TYPO3\CMS\Core\Resource\FileRepository::class);
            $fallbackUrl = $fileRepository->findFileReferenceByUid(intval($arguments['fallbackImg']))->getPublicUrl();
            $html .= '<img src="/' . $fallbackUrl . '" />';
        }

        $html .= '</canvas>';
        $html .= '</div>';
        $html .= '</div>';

        return $html;
    }

    /**
     * Converts the a CSV string to an array.
     *
     * Example:
     * csv2Array("label1,label2,label3\nvalue1,value2,value3") ->
     * [["label1", "label2", "label3"],[value1, value2, value3]]
     *
     * @param string $chartData - The data as CSV string.
     * @return array
     */
    public function csv2Array($chartData)
    {
        $data = [];

        // Split at new lines
        $records = preg_split('/$\R?^/m', $chartData);
        foreach ($records as $record) {
            $data[] = str_getcsv($record);
        }
        return $data;
    }

    /**
     * Build the correct datasets for each chart type.
     * For more info visit: http://www.chartjs.org/docs/
     *
     * @param array $chartData
     * @return void
     */
    public function getDatasetsFromCsv($chartData)
    {
        $dataRecords = $this->csv2Array($chartData);
        /**
         * Generate Labels
         */

        $this->data['labels'] = [];

        if ($this->type == 'pie' || $this->type == 'doughnut' || $this->type == 'polarArea') {
            $this->datasets = [new \stdClass];
            $this->datasets[0]->data = [];
            $this->datasets[0]->backgroundColor = [];


            if ($this->type != 'polarArea') {
                $this->datasets[0]->hoverBackgroundColor = [];
            } else {
                $this->datasets[0]->label = '';
            }

            for ($i = 0, $length = count($dataRecords); $i < $length; $i++) {
                $this->values[] = $dataRecords[$i][1];
                $this->addLabel($dataRecords[$i][0]);
            }
            $this->rowCount = $length;
        } else {
            for ($i = 0, $length = count($dataRecords); $i < $length - 1; $i++) {
                $this->values[] = array_slice($dataRecords[$i + 1], 1);
                $labels[] = trim($dataRecords[$i + 1][0]);
            }

            $this->rowCount = $length - 1;

            $this->labels = array_map('trim', $dataRecords[0]);
        }

        //Get colors from settings
        $colors = $this->arguments['colors'];
        $datasets = [];
        for ($i = 0; $i < $this->rowCount; $i++) {
            $datasetData = [];
            //Get current color
            if ($this->arguments['maxColors']) {
                $color = $colors[$i % $this->arguments['maxColors']];
            } else {
                $color = $colors[$i];
            }
            if (strpos($color, 'rgba') !== false) {
                $alpha = $this->chartUtility->getAlphaFromRgba($color);
            }
            $alpha = ($alpha) ? $alpha : 1;
            $alphaHover = ($alpha) ? $alpha - 0.1 : 0.7;
            switch ($this->type) {
                case 'pie':
                case 'doughnut':
                    $this->datasets[0]->data[] = (int) $this->values[$i];
                    $this->datasets[0]->backgroundColor[] = $this->chartUtility->hex2rgba($color, $alpha);
                    $this->datasets[0]->hoverBackgroundColor[] = $this->chartUtility->hex2rgba($color, $alphaHover);
                    break;

                case 'polarArea':
                    $this->datasets[0]->data[] = (int) $this->values[$i];
                    $this->datasets[0]->backgroundColor[] = $this->chartUtility->hex2rgba($color, $alpha);
                    break;

                case 'bar':
                case 'horizontalBar':
                    $specialAlpha = ($alpha) ? $alpha - 0.1 : 0.8;
                    $borderAlpha = ($alpha) ? $alpha - 0.1 : 0.7;
                    $dataset = new \stdClass;
                    $dataset->label = $labels[$i];
                    $dataset->backgroundColor = $this->chartUtility->hex2rgba($color, $alpha);
                    $dataset->borderColor = $this->chartUtility->hex2rgba($color, $borderAlpha);
                    $dataset->hoverBackgroundColor = $this->chartUtility->hex2rgba($color, $specialAlpha);
                    $dataset->hoverBorderColor = $this->chartUtility->hex2rgba($color, $specialAlpha);
                    $dataset->data = $this->values[$i];
                    $this->datasets[] = $dataset;
                    break;

                case 'line':
                case 'radar':
                    $specialAlpha = ($alpha) ? $alpha - 0.1 : 0.8;
                    $dataset = new \stdClass;
                    $dataset->label = $this->labels[$i];
                    $dataset->fill = false;
                    $dataset->backgroundColor = $this->chartUtility->hex2rgba($color, $alpha);
                    $dataset->borderColor = $this->chartUtility->hex2rgba($color, $specialAlpha);
                    $dataset->pointBackgroundColor = $this->chartUtility->hex2rgba($color, $specialAlpha);
                    $dataset->pointBorderColor = '#FFF';
                    $dataset->pointHoverBackgroundColor = '#FFF';
                    $dataset->pointHoverBorderColor = $this->chartUtility->hex2rgba($color, $specialAlpha);
                    $dataset->data = $this->values[$i];
                    $this->datasets[] = $dataset;
                    break;

                default:
                    break;
            }
        }
    }
}
